#=
Created on 03 Oct 2021

@author: David Doblas Jiménez
@email: daviddoji@pm.me

Solution for Problem 55 of Project Euler
https://projecteuler.net/problem=55
=#

using BenchmarkTools

function is_palindrome(num)
    return num == reverse(num)
end

function Problem55()
    #=
    If we take 47, reverse and add, 47 + 74 = 121, which is palindromic.

    Not all numbers produce palindromes so quickly. For example,

        349 + 943 = 1292,
        1292 + 2921 = 4213
        4213 + 3124 = 7337

    That is, 349 took three iterations to arrive at a palindrome.

    Although no one has proved it yet, it is thought that some numbers, like 196,
    never produce a palindrome. A number that never forms a palindrome through the
    reverse and add process is called a Lychrel number. Due to the theoretical nature
    of these numbers, and for the purpose of this problem, we shall assume that a
    number is Lychrel until proven otherwise. In addition you are given that for
    every number below ten-thousand, it will either:
    (i) become a palindrome in less than fifty iterations, or,
    (ii) no one, with all the computing power that exists, has managed so far to map
    it to a palindrome.

    In fact, 10677 is the first number to be shown to require over fifty iterations
    before producing a palindrome:

        4668731596684224866951378664 (53 iterations, 28-digits).

    Surprisingly, there are palindromic numbers that are themselves Lychrel numbers;
    the first example is 4994.

    How many Lychrel numbers are there below ten-thousand?
    =#

    ans = 0
    for n in 11:10_000
        num = n
        is_lychrel = true
        for it in 0:50
            num += parse(BigInt, reverse(string(num)))
            if is_palindrome(digits(num, base=10))
                is_lychrel = false
                break
            end
        end
        if is_lychrel
            ans += 1
        end
    end

    return ans
end


println("Time to evaluate Problem $(lpad(55, 3, "0")):")
@btime Problem55()
println("")
println("Result for Problem $(lpad(55, 3, "0")): ", Problem55())
